package pp.battleship.client.gui;

import com.jme3.math.FastMath;
import com.jme3.math.Quaternion;
import com.jme3.math.Vector3f;
import com.jme3.renderer.RenderManager;
import com.jme3.renderer.ViewPort;
import com.jme3.scene.Node;
import com.jme3.scene.control.AbstractControl;

/**
 * Control that handles the sinking effect for destroyed ships.
 * It will gradually move the ship downwards and then remove it from the scene.
 */
class SinkingControl extends AbstractControl {
    private static final float SINK_DURATION = 2f;  // Duration of the sinking animation
    private static final float SINK_SPEED = 0.6f;   // Speed at which the ship sinks
    private float elapsedTime = 0;

    private final Node shipNode;

    /**
     * Constructs a {@code SinkingControl} object with the shipNode to be to be sunk
     * @param shipNode the node to handeld
     */
    public SinkingControl(Node shipNode) {
        this.shipNode = shipNode;
    }
    float tiltAngle= 0;
    /**
     * Updated the Map to sink the ship
     * 
     * @param tpf time per frame
     */
    @Override
    protected void controlUpdate(float tpf) {
        // Update the sinking effect
        elapsedTime += tpf;

        // Move the ship down over time
        Vector3f currentPos = shipNode.getLocalTranslation();
        shipNode.setLocalTranslation(currentPos.x+SINK_SPEED*tpf, currentPos.y - SINK_SPEED * tpf*2, currentPos.z+SINK_SPEED*tpf*2);
        
        // Check if sinking duration has passed
        if (elapsedTime >= SINK_DURATION) {
            // Remove the ship from the scene
            shipNode.removeFromParent();
        }
        // Apply the tilt angle and rotation during sinking
        float sinkingTiltAngle = FastMath.DEG_TO_RAD * tiltAngle * tpf;
        float sinkingRotationAngle = FastMath.DEG_TO_RAD * 45f * tpf; // Additional rotation during sinking
        Quaternion tiltRotation = new Quaternion().fromAngles(-sinkingTiltAngle, sinkingRotationAngle, 0);
        spatial.setLocalRotation(tiltRotation);
    }

    @Override
    protected void controlRender(RenderManager rm, ViewPort vp) {
        // No rendering-related code needed
    }
}
